@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -106,6 +106,7 @@ void insert_accounting_structure(struct pkt_data *data)
         elem_acc->packet_counter += data->pkt_num;
         elem_acc->flow_counter += data->flo_num;
         elem_acc->bytes_counter += data->pkt_len;
+	elem_acc->tcp_flags |= data->tcp_flags;
         if (config.what_to_count & COUNT_CLASS) {
           elem_acc->packet_counter += data->cst.pa;
           elem_acc->bytes_counter += data->cst.ba;
@@ -126,6 +127,7 @@ void insert_accounting_structure(struct pkt_data *data)
         elem_acc->packet_counter += data->pkt_num;
         elem_acc->flow_counter += data->flo_num;
         elem_acc->bytes_counter += data->pkt_len;
+	elem_acc->tcp_flags |= data->tcp_flags;
 	if (config.what_to_count & COUNT_CLASS) {
 	  elem_acc->packet_counter += data->cst.pa;
 	  elem_acc->bytes_counter += data->cst.ba;
@@ -141,6 +143,7 @@ void insert_accounting_structure(struct pkt_data *data)
       elem_acc->packet_counter += data->pkt_num;
       elem_acc->flow_counter += data->flo_num;
       elem_acc->bytes_counter += data->pkt_len;
+      elem_acc->tcp_flags |= data->tcp_flags;
       elem_acc->signature = hash;
       if (config.what_to_count & COUNT_CLASS) {
         elem_acc->packet_counter += data->cst.pa;
@@ -190,6 +193,7 @@ void insert_accounting_structure(struct pkt_data *data)
       elem_acc->packet_counter += data->pkt_num;
       elem_acc->flow_counter += data->flo_num;
       elem_acc->bytes_counter += data->pkt_len;
+      elem_acc->tcp_flags = data->tcp_flags;
       elem_acc->signature = hash; 
       if (config.what_to_count & COUNT_CLASS) {
         elem_acc->packet_counter += data->cst.pa;
@@ -213,5 +217,6 @@ void reset_counters(struct acc *elem)
   elem->reset_flag = FALSE;
   elem->packet_counter = 0;
   elem->bytes_counter = 0;
+  elem->tcp_flags = 0;
   memcpy(&elem->rstamp, &cycle_stamp, sizeof(struct timeval));
 }
