@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -34,6 +34,7 @@ struct configuration {
   char *name;
   char *type;
   u_int32_t what_to_count;
+  int sock;
   int acct_type; 
   int data_type; 
   int pipe_size;
@@ -86,6 +87,8 @@ struct configuration {
   int nfacctd_time;
   int nfacctd_as;
   int sfacctd_renormalize;
+  int nfacctd_disable_checks;
+  int nfacctd_sql_log;
   int promisc; /* pcap_open_live() promisc parameter */
   char *clbuf; /* pcap filter */
   char *pcap_savefile;
@@ -106,9 +109,11 @@ struct configuration {
   char *ports_file;
   int refresh_maps;
   char *a_filter;
-  struct bpf_program bpfp_a_filter;
+  int bpfp_a_num;
+  struct bpf_program *bpfp_a_table[AGG_FILTER_ENTRIES];
   struct pretag_filter ptf;
   char *pre_tag_map;
+  int pre_tag_map_entries;
   int post_tag;
   int sampling_rate;
   char *syslog;
@@ -128,6 +133,7 @@ struct configuration {
   char *sfprobe_receiver;
   char *sfprobe_agentip;
   int sfprobe_agentsubid;
+	int flow_handling_threads;
 };
 
 struct plugin_type_entry {
