@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -152,6 +152,7 @@ int cfg_key_aggregate(char *filename, char *name, char *value_ptr)
     else if (!strcmp(count_token, "tag")) value |= COUNT_ID;
     else if (!strcmp(count_token, "flows")) value |= COUNT_FLOWS;
     else if (!strcmp(count_token, "class")) value |= COUNT_CLASS;
+    else if (!strcmp(count_token, "tcpflags")) value |= COUNT_TCPFLAGS;
     else Log(LOG_WARNING, "WARN ( %s ): ignoring unknown aggregation method: %s.\n", filename, count_token);
   }
 
@@ -1414,6 +1415,23 @@ int cfg_key_pre_tag_map(char *filename, char *name, char *value_ptr)
   return changes;
 }
 
+int cfg_key_pre_tag_map_entries(char *filename, char *name, char *value_ptr)
+{
+  struct plugins_list_entry *list = plugins_list;
+  int value, changes = 0;
+
+  value = atoi(value_ptr);
+  if (value <= 0) {
+    Log(LOG_ERR, "WARN ( %s ): 'pre_tag_map_entries' has to be > 0.\n", filename);
+    return ERR;
+  }
+
+  for (; list; list = list->next, changes++) list->cfg.pre_tag_map_entries = value;
+  if (name) Log(LOG_WARNING, "WARN ( %s ): plugin name not supported for key 'pre_tag_map_entries'. Globalized.\n", filename);
+
+  return changes;
+}
+
 int cfg_key_nfacctd_time_secs(char *filename, char *name, char *value_ptr)
 {
   struct plugins_list_entry *list = plugins_list;
@@ -1474,6 +1492,28 @@ int cfg_key_nfacctd_mcast_groups(char *filename, char *name, char *value_ptr)
   return changes;
 }
 
+int cfg_key_nfacctd_sql_log(char *filename, char *name, char *value_ptr)
+{
+  struct plugins_list_entry *list = plugins_list;
+  int value, changes = 0;
+
+  value = parse_truefalse(value_ptr);
+  if (value < 0) return ERR;
+
+  if (!name) for (; list; list = list->next, changes++) list->cfg.nfacctd_sql_log = value;
+  else {
+    for (; list; list = list->next) {
+      if (!strcmp(name, list->name)) {
+        list->cfg.nfacctd_sql_log = value;
+        changes++;
+	break;
+      }
+    }
+  }
+
+  return changes;
+}
+
 int cfg_key_pmacctd_force_frag_handling(char *filename, char *name, char *value_ptr)
 {
   struct plugins_list_entry *list = plugins_list;
@@ -1612,6 +1652,21 @@ int cfg_key_nfacctd_as_new(char *filename, char *name, char *value_ptr)
   return changes;
 }
 
+int cfg_key_nfacctd_disable_checks(char *filename, char *name, char *value_ptr)
+{
+  struct plugins_list_entry *list = plugins_list;
+  int value, changes = 0;
+
+  value = parse_truefalse(value_ptr);
+  if (value < 0) return ERR;
+
+  for (; list; list = list->next, changes++) list->cfg.nfacctd_disable_checks = value;
+  if (name) Log(LOG_WARNING, "WARN ( %s ): plugin name not supported for key '[ns]facctd_disable_checks'. Globalized.\n", filename);
+
+  return changes;
+}
+
+
 int cfg_key_classifiers(char *filename, char *name, char *value_ptr)
 {
   struct plugins_list_entry *list = plugins_list;
@@ -1852,6 +1907,29 @@ int cfg_key_sfprobe_agentsubid(char *filename, char *name, char *value_ptr)
   return changes;
 }
 
+int cfg_key_flow_handling_threads(char *filename, char *name, char *value_ptr)
+{
+  struct plugins_list_entry *list = plugins_list;
+  int changes = 0;
+  int value;
+
+  value = atoi(value_ptr);
+  if (!name) for (; list; list = list->next, changes++) list->cfg.flow_handling_threads = value;
+  else {
+    for (; list; list = list->next) {
+      if (!strcmp(name, list->name)) {
+        list->cfg.flow_handling_threads = value;
+        changes++;
+        break;
+      }
+    }
+  }
+
+  return changes;
+}
+
+
+
 void parse_time(char *filename, char *value, int *mu, int *howmany)
 {
   int k, j, len;
