@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 #include "../include/extract.h"
@@ -221,7 +221,9 @@ struct pkt_data {
   pm_counter_t pkt_len;
   pm_counter_t pkt_num;
   pm_counter_t flo_num;
-  u_int32_t pkt_time;
+  u_int32_t tcp_flags; /* XXX */
+  u_int32_t time_start;
+  u_int32_t time_end;
   struct class_st cst;
 };
 
@@ -230,8 +232,9 @@ struct pkt_payload {
   pm_counter_t sample_pool;
   pm_counter_t pkt_len;
   pm_counter_t pkt_num;
-  u_int32_t pkt_time;
+  u_int32_t time_start;
   pm_class_t class;
+  pm_id_t tag;
   struct host_addr src_ip;
   struct host_addr dst_ip;
 };
