@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -54,7 +54,7 @@ void usage_daemon(char *prog_name)
   printf("  -L  \tBind to the specified IP address\n");
   printf("  -l  \tListen on the specified UDP port\n");
   printf("  -f  \tLoad configuration from the specified file\n");
-  printf("  -c  \t[ src_mac | dst_mac | vlan | src_host | dst_host | src_net | dst_net | src_port | dst_port |\n\t tos | proto | src_as | dst_as | sum_mac | sum_host | sum_net | sum_as | sum_port | tag |\n\t flows | class | none ] \n\tAggregation string (DEFAULT: src_host)\n");
+  printf("  -c  \t[ src_mac | dst_mac | vlan | src_host | dst_host | src_net | dst_net | src_port | dst_port |\n\t tos | proto | src_as | dst_as | sum_mac | sum_host | sum_net | sum_as | sum_port | tag |\n\t flows | class | tcpflags | none ] \n\tAggregation string (DEFAULT: src_host)\n");
   printf("  -D  \tDaemonize\n"); 
   printf("  -n  \tPath to a file containing Network definitions\n");
   printf("  -o  \tPath to a file containing Port definitions\n");
@@ -69,7 +69,7 @@ void usage_daemon(char *prog_name)
   printf("  -s  \tMemory pool size\n");
   printf("\nPostgreSQL (-P pgsql)/MySQL (-P mysql)/SQLite (-P sqlite3) plugin options:\n");
   printf("  -r  \tRefresh time (in seconds)\n");
-  printf("  -v  \t[ 1 | 2 | 3 | 4 | 5 | 6 ] \n\tTable version\n");
+  printf("  -v  \t[ 1 | 2 | 3 | 4 | 5 | 6 | 7 ] \n\tTable version\n");
   printf("\n");
   printf("Examples:\n");
   printf("  Daemonize the process and write data into a MySQL database\n");
@@ -90,7 +90,7 @@ int main(int argc,char **argv, char **envp)
   struct packet_ptrs_vector pptrs;
   char config_file[SRVBUFLEN];
   unsigned char netflow_packet[NETFLOW_MSG_SIZE];
-  int logf, sd, rc, yes=1, allowed;
+  int logf, rc, yes=1, allowed;
   struct host_addr addr;
   struct hosts_table allow;
   struct id_table idt;
@@ -331,6 +331,8 @@ int main(int argc,char **argv, char **envp)
 	}
 	if (list->cfg.nfprobe_version == 9 && list->cfg.classifiers_path)
 	  list->cfg.what_to_count |= COUNT_CLASS;
+	if (list->cfg.nfprobe_version == 9 && list->cfg.pre_tag_map)
+	  list->cfg.what_to_count |= COUNT_ID;
 	if (list->cfg.nfprobe_version == 9)
 	  list->cfg.what_to_count |= COUNT_FLOWS;
 	list->cfg.what_to_count |= COUNT_COUNTERS;
@@ -342,6 +344,7 @@ int main(int argc,char **argv, char **envp)
 	req.bpf_filter = TRUE;
 	list->cfg.what_to_count = COUNT_PAYLOAD;
 	if (list->cfg.classifiers_path) list->cfg.what_to_count |= COUNT_CLASS;
+	if (list->cfg.pre_tag_map) list->cfg.what_to_count |= COUNT_ID;
 
 	list->cfg.data_type = PIPE_TYPE_PAYLOAD;
       }
@@ -409,22 +412,22 @@ int main(int argc,char **argv, char **envp)
   }
 
   /* socket creation */
-  sd = socket(((struct sockaddr *)&server)->sa_family, SOCK_DGRAM, 0);
-  if (sd < 0) {
+  config.sock = socket(((struct sockaddr *)&server)->sa_family, SOCK_DGRAM, 0);
+  if (config.sock < 0) {
     Log(LOG_ERR, "ERROR ( default/core ): socket() failed.\n");
     exit(1);
   }
 
   /* bind socket to port */
-  rc = Setsocksize(sd, SOL_SOCKET, SO_REUSEADDR, (char *)&yes, sizeof(yes));
+  rc = Setsocksize(config.sock, SOL_SOCKET, SO_REUSEADDR, (char *)&yes, sizeof(yes));
   if (rc < 0) Log(LOG_ERR, "WARN ( default/core ): Setsocksize() failed for SO_REUSEADDR.\n");
 
   if (config.pipe_size) {
-    rc = Setsocksize(sd, SOL_SOCKET, SO_RCVBUF, &config.pipe_size, sizeof(config.pipe_size));
+    rc = Setsocksize(config.sock, SOL_SOCKET, SO_RCVBUF, &config.pipe_size, sizeof(config.pipe_size));
     if (rc < 0) Log(LOG_ERR, "WARN ( default/core ): Setsocksize() failed for 'plugin_pipe_size' = '%d'.\n", config.pipe_size); 
   }
 
-  rc = bind(sd, (struct sockaddr *) &server, slen);
+  rc = bind(config.sock, (struct sockaddr *) &server, slen);
   if (rc < 0) {
     Log(LOG_ERR, "ERROR ( default/core): bind() to ip=%s port=%d/udp failed (errno: %d).\n", config.nfacctd_ip, config.nfacctd_port, errno);
     exit(1);
@@ -435,7 +438,7 @@ int main(int argc,char **argv, char **envp)
     if (mcast_groups[idx].family == AF_INET) { 
       memset(&multi_req4, 0, sizeof(multi_req4));
       multi_req4.imr_multiaddr.s_addr = mcast_groups[idx].address.ipv4.s_addr;
-      if (setsockopt(sd, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&multi_req4, sizeof(multi_req4)) < 0) {
+      if (setsockopt(config.sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&multi_req4, sizeof(multi_req4)) < 0) {
         Log(LOG_ERR, "ERROR: IPv4 multicast address - ADD membership failed.\n");
         exit(1);
       }
@@ -444,7 +447,7 @@ int main(int argc,char **argv, char **envp)
     if (mcast_groups[idx].family == AF_INET6) {
       memset(&multi_req6, 0, sizeof(multi_req6));
       ip6_addr_cpy(&multi_req6.ipv6mr_multiaddr, &mcast_groups[idx].address.ipv6); 
-      if (setsockopt(sd, IPPROTO_IPV6, IPV6_JOIN_GROUP, (char *)&multi_req6, sizeof(multi_req6)) < 0) {
+      if (setsockopt(config.sock, IPPROTO_IPV6, IPV6_JOIN_GROUP, (char *)&multi_req6, sizeof(multi_req6)) < 0) {
         Log(LOG_ERR, "ERROR: IPv6 multicast address - ADD membership failed.\n");
         exit(1);
       }
@@ -617,7 +620,7 @@ int main(int argc,char **argv, char **envp)
 
   /* Main loop */
   for(;;) {
-    ret = recvfrom(sd, netflow_packet, NETFLOW_MSG_SIZE, 0, (struct sockaddr *) &client, &clen);
+    ret = recvfrom(config.sock, netflow_packet, NETFLOW_MSG_SIZE, 0, (struct sockaddr *) &client, &clen);
 
     if (ret < 1) continue; /* we don't have enough data to decode the version */ 
 
