@@ -391,7 +391,8 @@ struct data_hdr_v9 {
 #define NF9_TEMPLATE_FLOWSET_ID         0
 #define NF9_OPTIONS_FLOWSET_ID          1
 #define NF9_MIN_RECORD_FLOWSET_ID       256
-#define NF9_MAX_DEFINED_FIELD		100
+// #define NF9_MAX_DEFINED_FIELD	100
+#define NF9_MAX_DEFINED_FIELD		210
 
 /* Flowset record types the we care about */
 #define NF9_IN_BYTES			1
@@ -448,6 +449,7 @@ struct data_hdr_v9 {
 #define NF9_MPLS_LABEL_10		79
 /* ... */
 #define NF9_CUST_CLASS			200
+#define NF9_CUST_TAG			201
 
 #define NF9_FTYPE_IPV4			0
 #define NF9_FTYPE_IPV6			1
