@@ -22,7 +22,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-/* $Id: netflow9.c,v 1.1.1.1 2006/11/19 15:16:07 paolo Exp $ */
+/* $Id: netflow9.c,v 1.2 2007/02/12 22:00:40 paolo Exp $ */
 
 #define __NFPROBE_NETFLOW9_C
 
@@ -86,6 +86,7 @@ struct NF9_DATA_FLOWSET_HEADER {
 #define NF9_MPLS_LABEL_1                70
 /* CUSTOM TYPES START HERE */
 #define NF9_CUST_CLASS			200
+#define NF9_CUST_TAG			201
 /* CUSTOM TYPES END HERE */
 
 /* Stuff pertaining to the templates that softflowd uses */
@@ -260,6 +261,15 @@ flow_to_flowset_class_handler(char *flowset, const struct FLOW *flow, int idx, i
   memcpy(flowset, buf, size);
 }
 
+static void
+flow_to_flowset_tag_handler(char *flowset, const struct FLOW *flow, int idx, int size)
+{
+  u_int16_t rec16;
+
+  rec16 = htons(flow->tag);
+  memcpy(flowset, &rec16, size);
+}
+
 static void
 nf9_init_template(void)
 {
@@ -416,6 +426,13 @@ nf9_init_template(void)
 	  v4_int_template.r[rcount].length = 16;
 	  rcount++;
 	}
+	if (config.nfprobe_what_to_count & COUNT_ID) {
+	  v4_template.r[rcount].type = htons(NF9_CUST_TAG);
+	  v4_template.r[rcount].length = htons(2);
+	  v4_int_template.r[rcount].handler = flow_to_flowset_tag_handler;
+	  v4_int_template.r[rcount].length = 2;
+	  rcount++;
+	}
 	v4_template.h.c.flowset_id = htons(0);
 	v4_template.h.c.length = htons( sizeof(struct NF9_TEMPLATE_FLOWSET_HEADER) + (sizeof(struct NF9_TEMPLATE_FLOWSET_RECORD) * rcount) );
 	v4_template.h.template_id = htons(NF9_SOFTFLOWD_V4_TEMPLATE_ID + config.nfprobe_id);
@@ -554,6 +571,13 @@ nf9_init_template(void)
 	  v6_int_template.r[rcount].length = 16;
 	  rcount++;
 	}
+        if (config.nfprobe_what_to_count & COUNT_ID) {
+	  v6_template.r[rcount].type = htons(NF9_CUST_TAG);
+	  v6_template.r[rcount].length = htons(2);
+	  v6_int_template.r[rcount].handler = flow_to_flowset_tag_handler;
+	  v6_int_template.r[rcount].length = 2;
+	  rcount++;
+	}
 	v6_template.h.c.flowset_id = htons(0);
 	v6_template.h.c.length = htons( sizeof(struct NF9_TEMPLATE_FLOWSET_HEADER) + (sizeof(struct NF9_TEMPLATE_FLOWSET_RECORD) * rcount) );
 	v6_template.h.template_id = htons(NF9_SOFTFLOWD_V6_TEMPLATE_ID + config.nfprobe_id);
