@@ -259,7 +259,9 @@ format_flow_brief(struct FLOW *flow)
  * swapped byte ordering for some primitives */ 
 void handle_hostbyteorder_packet(struct pkt_data *data)
 {
+#if defined HAVE_L2
   data->primitives.vlan_id = htons(data->primitives.vlan_id); 
+#endif
   data->primitives.src_port = htons(data->primitives.src_port); 
   data->primitives.dst_port = htons(data->primitives.dst_port); 
   data->primitives.src_as = htons(data->primitives.src_as); 
@@ -341,6 +343,7 @@ ipv4_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *ext
   flow->packets[ndx] = data->pkt_num;
   flow->flows[ndx] = data->flo_num;
   flow->class = p->class;
+  flow->tag = p->id;
 
   *isfrag = 0;
 
@@ -372,6 +375,7 @@ ipv6_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *ext
   flow->packets[ndx] = data->pkt_num; 
   flow->flows[ndx] = data->flo_num;
   flow->class = p->class;
+  flow->tag = p->id;
 
   *isfrag = 0;
 
@@ -549,6 +553,7 @@ process_packet(struct FLOWTRACK *ft, struct pkt_data *data, struct pkt_extras *e
     flow->tcp_flags[1] |= tmp.tcp_flags[1];
     flow->tos[1] = tmp.tos[1]; // XXX
     if (!flow->class) flow->class = tmp.class;
+    if (!flow->tag) flow->tag = tmp.tag;
   }
 	
   memcpy(&flow->flow_last, received_time, sizeof(flow->flow_last));
@@ -975,7 +980,7 @@ static void flow_cb(u_char *user_data, struct pkt_data *data, struct pkt_extras
   struct CB_CTXT *cb_ctxt = (struct CB_CTXT *)user_data;
   struct timeval tv;
 
-  tv.tv_sec = data->pkt_time;
+  tv.tv_sec = data->time_start;
   tv.tv_usec = 0; /* XXX */ 
   if (process_packet(cb_ctxt->ft, data, extras, &tv) == PP_MALLOC_FAIL) cb_ctxt->fatal = 1;
 }
@@ -1251,6 +1256,8 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   signal(SIGCHLD, ignore_falling_child);
 #endif
 	    
+  memset(&cb_ctxt, '\0', sizeof(cb_ctxt));
+
   init_flowtrack(&flowtrack);
 
   memset(&dest, '\0', sizeof(dest));
