@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -313,27 +313,29 @@ int PG_cache_dbop_copy(struct DBdesc *db, struct db_cache *cache_elem, struct in
 int PG_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct insert_data *idata)
 {
   PGresult *ret;
-  char *ptr_values, *ptr_where;
-  int num=0, have_flows=0;
+  char *ptr_values, *ptr_where, *ptr_set;
+  int num, have_flows=0;
 
   if (config.what_to_count & COUNT_FLOWS) have_flows = TRUE;
 
   /* constructing SQL query */
   ptr_where = where_clause;
   ptr_values = values_clause; 
+  ptr_set = set_clause;
   memset(where_clause, 0, sizeof(where_clause));
   memset(values_clause, 0, sizeof(values_clause));
+  memset(set_clause, 0, sizeof(set_clause));
 
-  while (num < idata->num_primitives) {
+  for (num = 0; num < idata->num_primitives; num++)
     (*where[num].handler)(cache_elem, idata, num, &ptr_values, &ptr_where);
-    num++;
-  }
+
+  for (num = 0; set[num].type; num++)
+    (*set[num].handler)(cache_elem, idata, num, &ptr_set, NULL);
 
   /* sending UPDATE query */
   if (!config.sql_dont_try_update) {
-    /* XXX: optimize time() usage */
-    if (have_flows) snprintf(sql_data, sizeof(sql_data), update_clause, cache_elem->packet_counter, cache_elem->bytes_counter, cache_elem->flows_counter, now);
-    else snprintf(sql_data, sizeof(sql_data), update_clause, cache_elem->packet_counter, cache_elem->bytes_counter, now);
+    strncpy(sql_data, update_clause, SPACELEFT(sql_data));
+    strncat(sql_data, set_clause, SPACELEFT(sql_data));
     strncat(sql_data, where_clause, SPACELEFT(sql_data));
 
     ret = PQexec(db->desc, sql_data);
@@ -414,7 +416,7 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
 
     if (config.sql_table_schema && idata->new_basetime) sql_create_table(bed.p, idata); 
   }
-  strncat(update_clause, set_clause, SPACELEFT(update_clause));
+  // strncat(update_clause, set_clause, SPACELEFT(update_clause));
 
   /* beginning DB transaction */
   (*sqlfunc_cbr.lock)(bed.p);
@@ -488,7 +490,7 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
 
 int PG_evaluate_history(int primitive)
 {
-  if (config.sql_history) {
+  if (config.sql_history || config.nfacctd_sql_log) {
     if (primitive) {
       strncat(copy_clause, ", ", SPACELEFT(copy_clause));
       strncat(insert_clause, ", ", SPACELEFT(insert_clause));
@@ -530,7 +532,7 @@ int PG_evaluate_history(int primitive)
 
 int PG_compose_static_queries()
 {
-  int primitives=0, have_flows=0, lock=0;
+  int primitives=0, set_primitives=0, have_flows=0, lock=0;
 
   if (config.what_to_count & COUNT_FLOWS || (config.sql_table_version >= 4 && !config.sql_optimize_clauses)) {
     config.what_to_count |= COUNT_FLOWS;
@@ -576,18 +578,42 @@ int PG_compose_static_queries()
 
   /* "UPDATE ... SET ..." stuff */
   snprintf(update_clause, sizeof(update_clause), "UPDATE %s ", config.sql_table);
-#if defined HAVE_64BIT_COUNTERS
-  strncpy(set_clause, "SET packets=packets+%llu, bytes=bytes+%llu", SPACELEFT(set_clause));
-  if (have_flows) strncat(set_clause, ", flows=flows+%llu", SPACELEFT(set_clause));
-#else
-  strncpy(set_clause, "SET packets=packets+%lu, bytes=bytes+%lu", SPACELEFT(set_clause));
-  if (have_flows) strncat(set_clause, ", flows=flows+%lu", SPACELEFT(set_clause));
-#endif
-  if (config.sql_history) {
-    if (!config.sql_history_since_epoch)
-      strncat(set_clause, ", stamp_updated=CURRENT_TIMESTAMP(0)", SPACELEFT(set_clause)); 
-    else
-      strncat(set_clause, ", stamp_updated=DATE_PART('epoch',NOW())::BIGINT", SPACELEFT(set_clause)); 
+
+  set_primitives = sql_compose_static_set(have_flows);
+
+  if (config.sql_history || config.nfacctd_sql_log) {
+    if (!config.nfacctd_sql_log) {
+      if (!config.sql_history_since_epoch) {
+	strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+	strncat(set[set_primitives].string, "stamp_updated=CURRENT_TIMESTAMP(0)", SPACELEFT(set[set_primitives].string)); 
+	set[set_primitives].type = TIMESTAMP;
+	set[set_primitives].handler = count_noop_setclause_handler;
+	set_primitives++;
+      }
+      else {
+	strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+	strncat(set[set_primitives].string, "stamp_updated=DATE_PART('epoch',NOW())::BIGINT", SPACELEFT(set[set_primitives].string));
+	set[set_primitives].type = TIMESTAMP;
+	set[set_primitives].handler = count_noop_setclause_handler;
+	set_primitives++;
+      }
+    }
+    else {
+      if (!config.sql_history_since_epoch) {
+	strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+	strncat(set[set_primitives].string, "stamp_updated=ABSTIME(%u)::Timestamp", SPACELEFT(set[set_primitives].string));
+	set[set_primitives].type = TIMESTAMP;
+	set[set_primitives].handler = count_timestamp_setclause_handler;
+	set_primitives++;
+      }
+      else {
+	strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+	strncat(set[set_primitives].string, "stamp_updated=%u", SPACELEFT(set[set_primitives].string));
+	set[set_primitives].type = TIMESTAMP;
+	set[set_primitives].handler = count_timestamp_setclause_handler;
+	set_primitives++;
+      }
+    }
   }
 
   /* values for COPY */
@@ -761,7 +787,8 @@ void PG_init_default_values(struct insert_data *idata)
     }
 
     if (typed) {
-      if (config.sql_table_version == 6) config.sql_table = pgsql_table_v6; 
+      if (config.sql_table_version == 7) config.sql_table = pgsql_table_v7;
+      else if (config.sql_table_version == 6) config.sql_table = pgsql_table_v6; 
       else if (config.sql_table_version == 5) {
         if (config.what_to_count & (COUNT_SRC_AS|COUNT_DST_AS|COUNT_SUM_AS)) config.sql_table = pgsql_table_as_v5;
         else config.sql_table = pgsql_table_v5;
@@ -784,7 +811,11 @@ void PG_init_default_values(struct insert_data *idata)
       }
     }
     else {
-      if (config.sql_table_version == 6) {
+      if (config.sql_table_version == 7) {
+	Log(LOG_WARNING, "WARN ( %s/%s ): Unified data are no longer supported. Switching to typed data.\n", config.name, config.type);
+	config.sql_table = pgsql_table_v7;
+      }
+      else if (config.sql_table_version == 6) {
 	Log(LOG_WARNING, "WARN ( %s/%s ): Unified data are no longer supported. Switching to typed data.\n", config.name, config.type);
 	config.sql_table = pgsql_table_v6;
       }
