@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -43,6 +43,11 @@ void load_plugins(struct plugin_requests *req)
   int l = sizeof(list->cfg.pipe_size);
   struct channels_list_entry *chptr = NULL;
 
+#if defined ENABLE_THREADS
+  channels_list_mutex = malloc(sizeof(pthread_mutex_t));
+  assert(channels_list_mutex);
+  pthread_mutex_init(channels_list_mutex, NULL);
+#endif
   init_random_seed(); 
   init_pipe_channels();
 
@@ -163,6 +168,7 @@ void load_plugins(struct plugin_requests *req)
 #else
 	signal(SIGCHLD, ignore_falling_child);
 #endif
+	close(config.sock);
 	close(list->pipe[1]);
 	(*list->type.func)(list->pipe[0], &list->cfg, chptr);
 	exit(0);
@@ -188,12 +194,13 @@ void exec_plugins(struct packet_ptrs *pptrs)
   char *bptr;
   int index;
 
+#if defined ENABLE_THREADS
+  pthread_mutex_lock(channels_list_mutex);
+#endif
   for (index = 0; channels_list[index].aggregation; index++) {
-    if (bpf_filter(channels_list[index].filter->bf_insns, pptrs->packet_ptr, pptrs->pkthdr->len,
-        pptrs->pkthdr->caplen) && (!channels_list[index].tag_filter.num ||
-	!evaluate_tags(&channels_list[index].tag_filter, pptrs->tag)) && 
-	pptrs->tag_dist && // !evaluate_sampling(&channels_list[index].s) &&
-	!check_shadow_status(pptrs, &channels_list[index])) {
+    if (evaluate_filters(&channels_list[index].agg_filter, pptrs->packet_ptr, pptrs->pkthdr) &&
+        (!channels_list[index].tag_filter.num || !evaluate_tags(&channels_list[index].tag_filter, pptrs->tag)) && 
+	pptrs->tag_dist && !check_shadow_status(pptrs, &channels_list[index])) {
       /* arranging buffer: supported primitives + packet total length */
 reprocess:
       channels_list[index].reprocess = FALSE;
@@ -259,6 +266,9 @@ reprocess:
       }
     }
   }
+#if defined ENABLE_THREADS
+  pthread_mutex_unlock(channels_list_mutex);
+#endif
 }
 
 struct channels_list_entry *insert_pipe_channel(struct configuration *cfg, int pipe)
@@ -271,7 +281,8 @@ struct channels_list_entry *insert_pipe_channel(struct configuration *cfg, int p
     if (!chptr->aggregation) { /* found room */
       chptr->aggregation = cfg->what_to_count;
       chptr->pipe = pipe; 
-      chptr->filter = &cfg->bpfp_a_filter; 
+      chptr->agg_filter.table = cfg->bpfp_a_table;
+      chptr->agg_filter.num = (int *) &cfg->bpfp_a_num; 
       chptr->bufsize = cfg->buffer_size;
       chptr->id = cfg->post_tag;
       if (cfg->sampling_rate) {
@@ -450,9 +461,10 @@ pm_counter_t take_simple_systematic_skip(pm_counter_t mean)
 }
 
 /* return value:
-   TRUE: discard the packet/flow
-   FALSE: Go on with the packet/flow either because a) we had a match or b) we are not filtering 
+   TRUE: We want it!
+   FALSE: Discard it!
 */
+
 int evaluate_tags(struct pretag_filter *filter, u_int16_t tag)
 {
   int index;
@@ -467,6 +479,23 @@ int evaluate_tags(struct pretag_filter *filter, u_int16_t tag)
   return TRUE;
 }
 
+/* return value:
+   TRUE: We want it!
+   FALSE: Discard it!
+*/
+int evaluate_filters(struct aggregate_filter *filter, char *pkt, struct pcap_pkthdr *pkthdr)
+{
+  int index;
+
+  if (*filter->num == 0) return TRUE;  /* no entries in the filter array: aggregate filtering disabled */
+
+  for (index = 0; index < *filter->num; index++) {
+    if (bpf_filter(filter->table[index]->bf_insns, pkt, pkthdr->len, pkthdr->caplen)) return TRUE; 
+  }
+
+  return FALSE;
+}
+
 void recollect_pipe_memory(struct channels_list_entry *mychptr)
 {
   struct channels_list_entry *chptr;
@@ -531,15 +560,26 @@ void load_plugin_filters(int link_type)
       if (list->cfg.a_filter) {
 	pcap_t *dev_desc;
 	bpf_u_int32 localnet, netmask;  /* pcap library stuff */
-	char errbuf[PCAP_ERRBUF_SIZE];
+	char errbuf[PCAP_ERRBUF_SIZE], *count_token;
+	int idx = 0;
 
 	dev_desc = pcap_open_dead(link_type, 128); /* 128 bytes should be long enough */
 
 	pcap_lookupnet(config.dev, &localnet, &netmask, errbuf);
-	if (pcap_compile(dev_desc, &list->cfg.bpfp_a_filter, list->cfg.a_filter, 0, netmask) < 0)
-	Log(LOG_WARNING, "WARN: %s\nWARN ( %s/%s ): aggregation filter disabled.\n",
-				pcap_geterr(dev_desc), config.name, config.type);
-	pcap_close(dev_desc);
+
+	list->cfg.bpfp_a_table[idx] = malloc(sizeof(struct bpf_program));
+	while ( (count_token = extract_token(&list->cfg.a_filter, ',')) && idx < AGG_FILTER_ENTRIES ) {
+	  if (pcap_compile(dev_desc, list->cfg.bpfp_a_table[idx], count_token, 0, netmask) < 0) {
+	    Log(LOG_WARNING, "WARN: %s\nWARN ( %s/%s ): aggregation filter disabled.\n",
+	    				pcap_geterr(dev_desc), list->cfg.name, list->cfg.type);
+	  }
+	  else {
+	    idx++;
+	    list->cfg.bpfp_a_table[idx] = malloc(sizeof(struct bpf_program));
+	  }
+	}
+
+	list->cfg.bpfp_a_num = idx;
       }
     }
     list = list->next;
