@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -56,6 +56,11 @@ struct sampling {
   skip_func sf;
 };
 
+struct aggregate_filter {
+  int *num;
+  struct bpf_program **table;
+};
+
 struct channels_list_entry {
   u_int32_t aggregation;
   u_int32_t buf;	/* buffer base */
@@ -65,20 +70,23 @@ struct channels_list_entry {
   struct ch_buf_hdr hdr;
   struct ch_status *status;
   ring_cleaner clean_func;
-  u_int8_t request;			/* does the plugin support on-request wakeup ? */
-  u_int8_t reprocess;			/* do we need to jump back for packet reprocessing ? */
+  u_int8_t request;					/* does the plugin support on-request wakeup ? */
+  u_int8_t reprocess;					/* do we need to jump back for packet reprocessing ? */
   int bufsize;		
   int same_aggregate;
   pkt_handler phandler[N_PRIMITIVES];
-  struct bpf_program *filter;
   int pipe;
-  u_int16_t id;				/* used to tag packets passing through the channel (post tag) */
-  struct pretag_filter tag_filter; 	/* used it to filter pre-tagged packets basing on their id */
+  u_int16_t id;						/* post-tagging id */
+  struct pretag_filter tag_filter; 			/* filter aggregates basing on their tag */
+  struct aggregate_filter agg_filter; 			/* filter aggregates basing on L2-L4 primitives */
   struct sampling s;
-  struct plugins_list_entry *plugin;	/* backpointer to the plugin the actual channel belongs to */
+  struct plugins_list_entry *plugin;			/* backpointer to the plugin the actual channel belongs to */
 };
 
 #if (defined __PLUGIN_HOOKS_C)
+#if (defined ENABLE_THREADS)
+pthread_mutex_t *channels_list_mutex;
+#endif
 extern struct channels_list_entry channels_list[MAX_N_PLUGINS];
 #endif
 
@@ -96,6 +104,7 @@ EXT void delete_pipe_channel(int);
 EXT void sort_pipe_channels();
 EXT void init_pipe_channels();
 EXT int evaluate_tags(struct pretag_filter *, u_int16_t);
+EXT int evaluate_filters(struct aggregate_filter *, char *, struct pcap_pkthdr *);
 EXT void recollect_pipe_memory(struct channels_list_entry *);
 EXT void init_random_seed();
 EXT void fill_pipe_buffer();
@@ -131,5 +140,7 @@ EXT void pgsql_plugin(int, struct configuration *, void *);
 EXT void sqlite3_plugin(int, struct configuration *, void *);
 #endif
 
+EXT void stats_plugin(int, struct configuration *, void *);
+
 EXT char *extract_token(char **, int);
 #undef EXT
