@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -265,6 +265,7 @@ static const struct _dictionary_line dictionary[] = {
   {"nfacctd_time_new", cfg_key_nfacctd_time_new},
   {"nfacctd_as_new", cfg_key_nfacctd_as_new},
   {"nfacctd_mcast_groups", cfg_key_nfacctd_mcast_groups},
+  {"nfacctd_sql_log", cfg_key_nfacctd_sql_log},
   {"pmacctd_force_frag_handling", cfg_key_pmacctd_force_frag_handling},
   {"pmacctd_frag_buffer_size", cfg_key_pmacctd_frag_buffer_size},
   {"pmacctd_flow_buffer_size", cfg_key_pmacctd_flow_buffer_size},
@@ -275,6 +276,7 @@ static const struct _dictionary_line dictionary[] = {
   {"nfacctd_id_file", cfg_key_pre_tag_map},	/* obsolete */
   {"pcap_savefile", cfg_key_pcap_savefile},
   {"pre_tag_map", cfg_key_pre_tag_map},	
+  {"pre_tag_map_entries", cfg_key_pre_tag_map_entries},	
   {"pre_tag_filter", cfg_key_pre_tag_filter},
   {"post_tag", cfg_key_post_tag},
   {"sampling_rate", cfg_key_sampling_rate},
@@ -284,6 +286,8 @@ static const struct _dictionary_line dictionary[] = {
   {"sfacctd_as_new", cfg_key_nfacctd_as_new},
   {"sfacctd_renormalize", cfg_key_sfacctd_renormalize},
   {"nfacctd_renormalize", cfg_key_sfacctd_renormalize},
+  {"nfacctd_disable_checks", cfg_key_nfacctd_disable_checks},
+  {"sfacctd_disable_checks", cfg_key_nfacctd_disable_checks},
   {"sfacctd_mcast_groups", cfg_key_nfacctd_mcast_groups},
   {"classifiers", cfg_key_classifiers},
   {"classifier_tentatives", cfg_key_classifier_tentatives},
@@ -299,6 +303,7 @@ static const struct _dictionary_line dictionary[] = {
   {"sfprobe_sampling_rate", cfg_key_sampling_rate},
   {"sfprobe_agentip", cfg_key_sfprobe_agentip},
   {"sfprobe_agentsubid", cfg_key_sfprobe_agentsubid},
+  {"flow_handling_threads", cfg_key_flow_handling_threads},
 
   {"", NULL},
 };
