@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -19,6 +19,9 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#ifndef _PMACCT_H_
+#define _PMACCT_H_
+
 /* includes */
 #ifdef HAVE_PCAP_PCAP_H
 #include <pcap/pcap.h>
@@ -32,6 +35,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <assert.h>
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h> 
@@ -289,3 +293,4 @@ extern pid_t failed_plugins[MAX_N_PLUGINS]; /* plugins failed during startup pha
 extern u_char dummy_tlhdr[16];
 #endif
 
+#endif /* _PMACCT_H_ */
