@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -32,6 +32,7 @@
 #include "ip_frag.h"
 #include "ip_flow.h"
 #include "net_aggr.h"
+#include "thread_pool.h"
 
 /* variables to be exported away */
 int debug;
@@ -52,7 +53,7 @@ void usage_daemon(char *prog_name)
   printf("\nGeneral options:\n");
   printf("  -h  \tShow this page\n");
   printf("  -f  \tLoad configuration from the specified file\n");
-  printf("  -c  \t[ src_mac | dst_mac | vlan | src_host | dst_host | src_net | dst_net | src_port | dst_port |\n\t proto | tos | src_as | dst_as | sum_mac | sum_host | sum_net | sum_as | sum_port | tag |\n\t flows | class | none ] \n\tAggregation string (DEFAULT: src_host)\n");
+  printf("  -c  \t[ src_mac | dst_mac | vlan | src_host | dst_host | src_net | dst_net | src_port | dst_port |\n\t proto | tos | src_as | dst_as | sum_mac | sum_host | sum_net | sum_as | sum_port | tag |\n\t flows | class | tcpflags | none ] \n\tAggregation string (DEFAULT: src_host)\n");
   printf("  -D  \tDaemonize\n"); 
   printf("  -N  \tDisable promiscuous mode\n");
   printf("  -n  \tPath to a file containing Network definitions\n");
@@ -72,7 +73,7 @@ void usage_daemon(char *prog_name)
   printf("  -s  \tMemory pool size\n");
   printf("\nPostgreSQL (-P pgsql)/MySQL (-P mysql)/SQLite (-P sqlite3) plugin options:\n");
   printf("  -r  \tRefresh time (in seconds)\n");
-  printf("  -v  \t[ 1 | 2 | 3 | 4 | 5 | 6 ] \n\tTable version\n");
+  printf("  -v  \t[ 1 | 2 | 3 | 4 | 5 | 6 | 7 ] \n\tTable version\n");
   printf("\n");
   printf("Examples:\n");
   printf("  Daemonize the process; listen on eth0; write stats in a MySQL database\n"); 
@@ -335,6 +336,8 @@ int main(int argc,char **argv, char **envp)
 	  list->cfg.what_to_count |= COUNT_CLASS; 
 	  config.handle_flows = TRUE;
 	}
+	if (list->cfg.nfprobe_version == 9 && list->cfg.pre_tag_map)
+	  list->cfg.what_to_count |= COUNT_ID;
 	list->cfg.what_to_count |= COUNT_COUNTERS;
 
 	list->cfg.data_type = PIPE_TYPE_METADATA;
@@ -349,6 +352,7 @@ int main(int argc,char **argv, char **envp)
 	  config.handle_fragments = TRUE;
 	  config.handle_flows = TRUE;
 	}
+	if (list->cfg.pre_tag_map) list->cfg.what_to_count |= COUNT_ID;
 
 	list->cfg.data_type = PIPE_TYPE_PAYLOAD;
       }
@@ -358,7 +362,7 @@ int main(int argc,char **argv, char **envp)
 	  exit(1);
         }
 	evaluate_sums(&list->cfg.what_to_count, list->name, list->type.string);
-	if (list->cfg.what_to_count & (COUNT_SRC_PORT|COUNT_DST_PORT|COUNT_SUM_PORT))
+	if (list->cfg.what_to_count & (COUNT_SRC_PORT|COUNT_DST_PORT|COUNT_SUM_PORT|COUNT_TCPFLAGS))
 	  config.handle_fragments = TRUE;
 	if (list->cfg.what_to_count & COUNT_FLOWS) {
 	  config.handle_fragments = TRUE;
@@ -391,6 +395,10 @@ int main(int argc,char **argv, char **envp)
     list = list->next;
   }
 
+#if defined ENABLE_THREADS
+  if (!config.flow_handling_threads) config.flow_handling_threads = DEFAULT_TH_NUM; 
+#endif
+
   /* plugins glue: creation (since 094) */
   if (config.classifiers_path) {
     init_classifiers(config.classifiers_path);
@@ -526,7 +534,10 @@ int main(int argc,char **argv, char **envp)
 
   /* When reading packets from a savefile, things are lightning fast; we will sit 
      here just few seconds, thus allowing plugins to complete their startup operations */ 
-  if (config.pcap_savefile) sleep(2);
+  if (config.pcap_savefile) {
+    Log(LOG_INFO, "INFO ( default/core ): PCAP capture file, sleeping for 2 seconds\n");
+    sleep(2);
+  }
 
   /* Main loop: if pcap_loop() exits maybe an error occurred; we will try closing
      and reopening again our listening device */
@@ -546,6 +557,7 @@ int main(int argc,char **argv, char **envp)
     if (config.pcap_savefile) {
       if (config.sf_wait) {
 	fill_pipe_buffer();
+	Log(LOG_INFO, "INFO ( default/core ): finished reading PCAP capture file\n");
 	wait(NULL);
       }
       stop_all_childs();
@@ -574,7 +586,13 @@ void pcap_cb(u_char *user, const struct pcap_pkthdr *pkthdr, const u_char *buf)
     if (pptrs.iph_ptr) {
       if ((*pptrs.l3_handler)(&pptrs)) {
 	if (config.pre_tag_map) pptrs.tag = PM_find_id(&pptrs);
+#ifdef ENABLE_THREADS
+	/* If we are not calling t_ip_flow_handler(), ie. not using classification
+	   or counting flows, we still need to give a kick to exec_plugins() here */ 
+	if ( !(config.what_to_count & COUNT_FLOWS) ) exec_plugins(&pptrs); 
+#else
 	exec_plugins(&pptrs); 
+#endif
       }
     }
   }
@@ -653,7 +671,11 @@ int ip_handler(register struct packet_ptrs *pptrs)
 	if (((struct my_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_ACK && pptrs->tcp_flags) pptrs->tcp_flags |= TH_ACK; 
       }
 
+#if defined ENABLE_THREADS
+      t_ip_flow_handler(pptrs);
+#else
       ip_flow_handler(pptrs);
+#endif
     }
 
     /* XXX: optimize/short circuit here! */
@@ -768,7 +790,11 @@ int ip6_handler(register struct packet_ptrs *pptrs)
         if (((struct my_tcphdr *)pptrs->tlh_ptr)->th_flags & TH_ACK && pptrs->tcp_flags) pptrs->tcp_flags |= TH_ACK;
       }
 
+#if defined ENABLE_THREADS
+      t_ip_flow6_handler(pptrs);
+#else
       ip_flow6_handler(pptrs);
+#endif
     }
 
     /* XXX: optimize/short circuit here! */
