@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -28,13 +28,15 @@
 #include "pretag_handlers.h"
 #include "pretag-data.h"
 
+int pre_tag_map_allocated = FALSE;
+
 void load_id_file(int acct_type, char *filename, struct id_table *t, struct plugin_requests *req)
 {
   struct id_table tmp;
   struct id_entry *ptr, *ptr2;
   FILE *file;
   char buf[SRVBUFLEN];
-  int v4_num = 0, x, tot_lines = 0, err, index, label_solved;
+  int v4_num = 0, x, tot_lines = 0, err, index, label_solved, sz;
   struct stat st;
 
 #if defined ENABLE_IPV6
@@ -47,6 +49,8 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 
   memset(&st, 0, sizeof(st));
 
+  if (!config.pre_tag_map_entries) config.pre_tag_map_entries = MAX_PRETAG_MAP_ENTRIES;
+
   if (filename) {
     if ((file = fopen(filename, "r")) == NULL) {
       Log(LOG_ERR, "ERROR: Pre-Tagging map '%s' not found.\n", filename);
@@ -56,10 +60,20 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
     memset(t, 0, sizeof(struct id_table));
     memset(&tmp, 0, sizeof(struct id_table));
 
+    sz = sizeof(struct id_entry)*config.pre_tag_map_entries;
+    if (!pre_tag_map_allocated) {
+      tmp.e = (struct id_entry *) malloc(sz);
+      t->e = (struct id_entry *) malloc(sz);
+      pre_tag_map_allocated = TRUE;
+    }
+
+    memset(tmp.e, 0, sz);
+    memset(t->e, 0, sz);
+
     /* first stage: reading Agent ID file and arranging it in a temporary memory table */
     while (!feof(file)) {
       tot_lines++;
-      if (tmp.num >= MAX_PRETAG_MAP_ENTRIES) break; /* XXX: we shouldn't exit silently */
+      if (tmp.num >= config.pre_tag_map_entries) break; /* XXX: we shouldn't exit silently */
       memset(buf, 0, SRVBUFLEN);
       if (fgets(buf, SRVBUFLEN, file)) {
         if (!iscomment(buf) && !isblankline(buf)) {
