@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -87,7 +87,7 @@ struct id_table {
   struct id_entry *ipv6_base;
   unsigned short int ipv6_num;
 #endif
-  struct id_entry e[MAX_PRETAG_MAP_ENTRIES];
+  struct id_entry *e;
   time_t timestamp;
 };
 
