@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -330,6 +330,7 @@ void P_cache_insert(struct pkt_data *data)
     cache_ptr->packet_counter = data->pkt_num;
     cache_ptr->flow_counter = data->flo_num;
     cache_ptr->bytes_counter = data->pkt_len;
+    cache_ptr->tcp_flags = data->tcp_flags;
     if (config.what_to_count & COUNT_CLASS) {
       cache_ptr->bytes_counter += data->cst.ba;
       cache_ptr->packet_counter += data->cst.pa;
@@ -343,6 +344,7 @@ void P_cache_insert(struct pkt_data *data)
       cache_ptr->packet_counter += data->pkt_num;
       cache_ptr->flow_counter += data->flo_num;
       cache_ptr->bytes_counter += data->pkt_len;
+      cache_ptr->tcp_flags |= data->tcp_flags;
       if (config.what_to_count & COUNT_CLASS) {
         cache_ptr->bytes_counter += data->cst.ba;
         cache_ptr->packet_counter += data->cst.pa;
@@ -354,6 +356,7 @@ void P_cache_insert(struct pkt_data *data)
       cache_ptr->packet_counter = data->pkt_num;
       cache_ptr->flow_counter = data->flo_num;
       cache_ptr->bytes_counter = data->pkt_len;
+      cache_ptr->tcp_flags = data->tcp_flags;
       if (config.what_to_count & COUNT_CLASS) {
         cache_ptr->bytes_counter += data->cst.ba;
         cache_ptr->packet_counter += data->cst.pa;
@@ -428,6 +431,7 @@ void P_cache_purge(struct chained_cache *queue[], int index)
 #endif
     printf("%-5d     ", data->src_port);
     printf("%-5d     ", data->dst_port);
+    printf("%-3d        ", queue[j]->tcp_flags);
     printf("%-10s  ", _protocols[data->proto].name);
     printf("%-3d    ", data->tos);
 #if defined HAVE_64BIT_COUNTERS
@@ -464,6 +468,7 @@ void P_write_stats_header()
 #endif
   printf("SRC_PORT  ");
   printf("DST_PORT  ");
+  printf("TCP_FLAGS  ");
   printf("PROTOCOL    ");
   printf("TOS    ");
 #if defined HAVE_64BIT_COUNTERS
