@@ -260,6 +260,10 @@ typedef struct _SFLExtended_classification {
   pm_class_t class;
 } SFLExtended_classification;
 
+typedef struct _SFLExtended_tag {
+  pm_id_t tag;
+} SFLExtended_tag;
+
 enum SFLFlow_type_tag {
   /* enterprise = 0, format = ... */
   SFLFLOW_HEADER    = 1,      /* Packet headers are sampled */
@@ -280,6 +284,7 @@ enum SFLFlow_type_tag {
   SFLFLOW_EX_VLAN_TUNNEL  = 1012,   /* VLAN stack */
   /* enterprise = 8800 pmacct */
   SFLFLOW_EX_CLASS        = (8800 << 12) + 1,
+  SFLFLOW_EX_TAG	  = (8800 << 12) + 2,
 };
 
 typedef union _SFLFlow_type {
@@ -300,6 +305,7 @@ typedef union _SFLFlow_type {
   SFLExtended_mpls_LDP_FEC mpls_ldp_fec;
   SFLExtended_vlan_tunnel vlan_tunnel;
   SFLExtended_classification class;
+  SFLExtended_tag tag;
 } SFLFlow_type;
 
 typedef struct _SFLFlow_sample_element {
