@@ -251,7 +251,7 @@ static void init_agent(SflSp *sp)
 
 static void readPacket(SflSp *sp, struct pkt_payload *hdr, const unsigned char *buf)
 {
-  SFLFlow_sample_element hdrElem, classHdrElem, gatewayHdrElem;
+  SFLFlow_sample_element hdrElem, classHdrElem, gatewayHdrElem, tagHdrElem;
   SFLExtended_as_path_segment as_path_segment;
   u_int32_t frame_len, header_len;
   int direction, sampledPackets;
@@ -348,6 +348,13 @@ static void readPacket(SflSp *sp, struct pkt_payload *hdr, const unsigned char *
 	SFLADD_ELEMENT(&fs, &classHdrElem);
       }
 
+      if (config.what_to_count & COUNT_ID) {
+        memset(&tagHdrElem, 0, sizeof(tagHdrElem));
+        tagHdrElem.tag = SFLFLOW_EX_TAG;
+        tagHdrElem.flowType.tag.tag = hdr->tag;
+        SFLADD_ELEMENT(&fs, &tagHdrElem);
+      }
+
       /*
          Extended gateway is meant to have a broad range of
          informations; we will fill in only infos pertaining
