@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -102,6 +102,11 @@ void my_sigint_handler(int signum)
 
   if (config.syslog) closelog();
 
+  /* We are about to exit, but it may take a while - because of the
+     wait() call. Let's release collector's socket to improve turn-
+     around times when restarting the daemon */
+  if (config.acct_type == ACCT_NF || config.acct_type == ACCT_SF) close(config.sock);
+
 #if defined (IRIX) || (SOLARIS)
   signal(SIGCHLD, SIG_IGN);
 #else
