@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -79,6 +79,7 @@ void sql_init_global_buffers()
   memset(insert_clause, 0, sizeof(insert_clause));
   memset(where, 0, sizeof(where));
   memset(values, 0, sizeof(values));
+  memset(set, 0, sizeof(set));
   memset(&lru_head, 0, sizeof(lru_head));
   lru_tail = &lru_head;
 
@@ -98,8 +99,11 @@ void sql_init_default_values()
   /* being the first routine to be called by each SQL plugin, this is
      also the place for some initial common configuration consistency
      check */ 
-  if (config.what_to_count & COUNT_CLASS && config.sql_recovery_logfile) {
-    Log(LOG_ERR, "ERROR: Recovery logfile and classifiers are mutually exclusive. Try with a backup DB.\n");
+  if ( (config.what_to_count & COUNT_CLASS ||
+	config.what_to_count & COUNT_TCPFLAGS ||
+	config.nfacctd_sql_log) &&
+       config.sql_recovery_logfile) {
+    Log(LOG_ERR, "ERROR: sql_recovery_logfile is not compatible with: classifiers, TCP flags and nfacctd_sql_log. Try configuring a backup DB.\n");
     exit_plugin(1);
   }
 
@@ -107,6 +111,7 @@ void sql_init_default_values()
   if (!config.sql_table_version) config.sql_table_version = DEFAULT_SQL_TABLE_VERSION;
   if (!config.sql_cache_entries) config.sql_cache_entries = CACHE_ENTRIES;
   if (!config.sql_max_writers) config.sql_max_writers = DEFAULT_SQL_WRITERS_NO;
+  if (config.nfacctd_sql_log && config.acct_type != ACCT_NF) config.nfacctd_sql_log = FALSE; 
 
   if (config.sql_aggressive_classification) {
     if (config.acct_type == ACCT_PM && config.what_to_count & COUNT_CLASS);
@@ -118,6 +123,7 @@ void sql_init_default_values()
   qq_size = config.sql_cache_entries+(config.sql_refresh_time*REASONABLE_NUMBER);
   pp_size = sizeof(struct pkt_primitives);
   dbc_size = sizeof(struct db_cache);
+  glob_nfacctd_sql_log = config.nfacctd_sql_log;
 
   memset(&sql_writers, 0, sizeof(sql_writers));
 }
@@ -310,7 +316,7 @@ struct db_cache *sql_cache_search(struct pkt_primitives *data, time_t basetime)
       /* additional check: pkt_primitives */
       if (!memcmp(&Cursor->primitives, data, sizeof(struct pkt_primitives))) {
         /* additional check: time */
-        if ((Cursor->basetime < basetime) && (config.sql_history))
+        if ((Cursor->basetime < basetime) && (config.sql_history || config.nfacctd_sql_log))
           goto follow_chain;
         else return Cursor;
       }
@@ -328,15 +334,15 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
   struct pkt_primitives *srcdst = &data->primitives;
   struct db_cache *Cursor, *newElem, *SafePtr = NULL, *staleElem = NULL;
 
-  if (data->pkt_time && config.sql_history) {
-    while (basetime > data->pkt_time) {
+  if (data->time_start && config.sql_history) {
+    while (basetime > data->time_start) {
       if (config.sql_history != COUNT_MONTHLY) basetime -= timeslot;
       else {
         timeslot = calc_monthly_timeslot(basetime, config.sql_history_howmany, SUB);
         basetime -= timeslot;
       }
     }
-    while ((basetime+timeslot) < data->pkt_time) {
+    while ((basetime+timeslot) < data->time_start) {
       if (config.sql_history != COUNT_MONTHLY) basetime += timeslot;
       else {
         basetime += timeslot;
@@ -414,7 +420,7 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
       /* additional check: pkt_primitives */
       if (!memcmp(&Cursor->primitives, srcdst, sizeof(struct pkt_primitives))) {
         /* additional check: time */
-        if ((Cursor->basetime < basetime) && (config.sql_history)) {
+        if ((Cursor->basetime < basetime) && (config.sql_history || config.nfacctd_sql_log)) {
           if (!staleElem && Cursor->chained) staleElem = Cursor;
           goto follow_chain;
         }
@@ -442,6 +448,7 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
   Cursor->packet_counter = data->pkt_num;
   Cursor->flows_counter = data->flo_num;
   Cursor->bytes_counter = data->pkt_len;
+  Cursor->tcp_flags = data->tcp_flags;
   if (config.what_to_count & COUNT_CLASS) {
     Cursor->bytes_counter += data->cst.ba;
     Cursor->packet_counter += data->cst.pa;
@@ -449,7 +456,14 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
     Cursor->tentatives = data->cst.tentatives;
   }
   Cursor->valid = SQL_CACHE_INUSE;
-  Cursor->basetime = basetime;
+  if (!config.nfacctd_sql_log) {
+    Cursor->basetime = basetime;
+    Cursor->endtime = 0;
+  }
+  else {
+    Cursor->basetime = data->time_start;
+    Cursor->endtime = data->time_end;
+  }
   Cursor->start_tag = idata->now;
   Cursor->lru_tag = idata->now;
   Cursor->signature = idata->hash;
@@ -462,6 +476,7 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
   Cursor->packet_counter += data->pkt_num;
   Cursor->flows_counter += data->flo_num;
   Cursor->bytes_counter += data->pkt_len;
+  Cursor->tcp_flags |= data->tcp_flags;
   if (config.what_to_count & COUNT_CLASS) {
     Cursor->bytes_counter += data->cst.ba;
     Cursor->packet_counter += data->cst.pa;
@@ -473,20 +488,23 @@ void sql_cache_insert(struct pkt_data *data, struct insert_data *idata)
   safe_action:
   Log(LOG_DEBUG, "DEBUG ( %s/%s ): purging process (CAUSE: safe action)\n", config.name, config.type);
 
+  if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, idata); 
   switch (fork()) {
   case 0: /* Child */
     signal(SIGINT, SIG_IGN);
     signal(SIGHUP, SIG_IGN);
     pm_setproctitle("%s [%s]", "SQL Plugin -- DB Writer (urgent)", config.name);
 
-    (*sqlfunc_cbr.connect)(&p, config.sql_host);
-    (*sqlfunc_cbr.purge)(queries_queue, qq_ptr, idata);
-    (*sqlfunc_cbr.close)(&bed);
+    if (qq_ptr && sql_writers.flags != CHLD_ALERT) {
+      if (sql_writers.flags == CHLD_WARNING) sql_db_fail(&p);
+      (*sqlfunc_cbr.connect)(&p, config.sql_host);
+      (*sqlfunc_cbr.purge)(queries_queue, qq_ptr, idata);
+      (*sqlfunc_cbr.close)(&bed);
+    }
+
     exit(0);
   default: /* Parent */
-    sql_cache_flush(queries_queue, qq_ptr, idata);
     qq_ptr = pqq_ptr;
-    pqq_ptr = 0;
     memcpy(queries_queue, pending_queries_queue, sizeof(queries_queue));
     break;
   }
@@ -704,7 +722,7 @@ int sql_evaluate_primitives(int primitive)
 
     if (config.what_to_count & COUNT_SUM_PORT) what_to_count |= COUNT_SUM_PORT;
 
-    what_to_count |= COUNT_SRC_PORT|COUNT_DST_PORT|COUNT_IP_PROTO|COUNT_ID|COUNT_CLASS|COUNT_VLAN|COUNT_IP_TOS;
+    what_to_count |= COUNT_SRC_PORT|COUNT_DST_PORT|COUNT_TCPFLAGS|COUNT_IP_PROTO|COUNT_ID|COUNT_CLASS|COUNT_VLAN|COUNT_IP_TOS;
   }
 
   /* 1st part: arranging pointers to an opaque structure and 
@@ -893,6 +911,31 @@ int sql_evaluate_primitives(int primitive)
     primitive++;
   }
 
+  if (what_to_count & COUNT_TCPFLAGS) {
+    int count_it = FALSE;
+
+    if ((config.sql_table_version < 7) && !assume_custom_table) {
+      if (config.what_to_count & COUNT_TCPFLAGS) {
+        Log(LOG_ERR, "ERROR ( %s/%s ): The use of TCP flags accounting requires SQL table v7. Exiting.\n", config.name, config.type);
+	exit_plugin(1);
+      }
+      else what_to_count ^= COUNT_TCPFLAGS;
+    }
+    else count_it = TRUE;
+
+    if (count_it) {
+      if (primitive) {
+	strncat(insert_clause, ", ", SPACELEFT(insert_clause));
+	strncat(values[primitive].string, ", ", sizeof(values[primitive].string));
+      }
+      strncat(insert_clause, "tcp_flags", SPACELEFT(insert_clause));
+      strncat(values[primitive].string, "%u", SPACELEFT(values[primitive].string));
+      values[primitive].type = where[primitive].type = COUNT_TCPFLAGS;
+      values[primitive].handler = where[primitive].handler = count_tcpflags_handler;
+      primitive++;
+    }
+  }
+
   if (what_to_count & COUNT_IP_TOS) {
     int count_it = FALSE;
 
@@ -1277,3 +1320,45 @@ int sql_select_locking_style(char *lock)
   return PM_LOCK_EXCLUSIVE;
 }
 
+int sql_compose_static_set(int have_flows)
+{
+  int set_primitives=0;
+
+#if defined HAVE_64BIT_COUNTERS
+  strncpy(set[set_primitives].string, "SET packets=packets+%llu, bytes=bytes+%llu", SPACELEFT(set[set_primitives].string));
+  set[set_primitives].type = COUNT_COUNTERS;
+  set[set_primitives].handler = count_counters_setclause_handler;
+  set_primitives++;
+
+  if (have_flows) {
+    strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+    strncat(set[set_primitives].string, "flows=flows+%llu", SPACELEFT(set[set_primitives].string));
+    set[set_primitives].type = COUNT_FLOWS;
+    set[set_primitives].handler = count_flows_setclause_handler;
+    set_primitives++;
+  }
+#else
+  strncpy(set[set_primitives].string, "SET packets=packets+%u, bytes=bytes+%u", SPACELEFT(set[set_primitives].string));
+  set[set_primitives].type = COUNT_COUNTERS;
+  set[set_primitives].handler = count_counters_setclause_handler;
+  set_primitives++;
+
+  if (have_flows) {
+    strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+    strncat(set[set_primitives].string, "flows=flows+%u", SPACELEFT(set[set_primitives].string));
+    set[set_primitives].type = COUNT_FLOWS;
+    set[set_primitives].handler = count_flows_setclause_handler;
+    set_primitives++;
+  }
+#endif
+
+  if (config.what_to_count & COUNT_TCPFLAGS) {
+    strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+    strncat(set[set_primitives].string, "tcp_flags=tcp_flags|%u", SPACELEFT(set[set_primitives].string));
+    set[set_primitives].type = COUNT_TCPFLAGS;
+    set[set_primitives].handler = count_tcpflags_setclause_handler;
+    set_primitives++;
+  }
+
+  return set_primitives;
+}
