@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -98,8 +98,10 @@ struct db_cache {
   pm_counter_t bytes_counter;
   pm_counter_t packet_counter;
   pm_counter_t flows_counter;
+  u_int32_t tcp_flags;
   u_int8_t tentatives;	/* support to classifiers: tentatives remaining */
   time_t basetime;
+  time_t endtime;
   short int valid;
   unsigned int signature;
   u_int8_t chained;
@@ -203,9 +205,16 @@ EXT void count_timestamp_handler(const struct db_cache *, const struct insert_da
 EXT void count_copy_timestamp_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
 EXT void count_id_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
 EXT void count_class_id_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
+EXT void count_tcpflags_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
 EXT void fake_mac_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
 EXT void fake_host_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
 EXT void fake_as_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
+
+EXT void count_counters_setclause_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
+EXT void count_flows_setclause_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
+EXT void count_timestamp_setclause_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
+EXT void count_tcpflags_setclause_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
+EXT void count_noop_setclause_handler(const struct db_cache *, const struct insert_data *, int, char **, char **);
 #undef EXT
 
 #if (defined __SQL_COMMON_C)
@@ -241,6 +250,7 @@ EXT FILE *sql_file_open(const char *, const char *, const struct insert_data *);
 EXT void sql_create_table(struct DBdesc *, struct insert_data *);
 EXT void sql_invalidate_shadow_entries(struct db_cache *[], int *);
 EXT int sql_select_locking_style(char *);
+EXT int sql_compose_static_set(int); 
 
 EXT void sql_sum_host_insert(struct pkt_data *, struct insert_data *);
 EXT void sql_sum_port_insert(struct pkt_data *, struct insert_data *);
@@ -278,10 +288,12 @@ EXT struct db_cache lru_head, *lru_tail;
 EXT struct frags where[N_PRIMITIVES+2];
 EXT struct frags values[N_PRIMITIVES+2];
 EXT struct frags copy_values[N_PRIMITIVES+2];
+EXT struct frags set[N_PRIMITIVES+2];
 EXT int glob_num_primitives; /* last resort for signal handling */
 EXT int glob_basetime; /* last resort for signal handling */
 EXT int glob_new_basetime; /* last resort for signal handling */
 EXT int glob_dyn_table; /* last resort for signal handling */
+EXT int glob_nfacctd_sql_log; /* last resort for sql handlers */
 
 EXT struct sqlfunc_cb_registry sqlfunc_cbr; 
 EXT void (*insert_func)(struct pkt_data *, struct insert_data *);
