@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -264,7 +264,7 @@ void sqlite3_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
 int SQLI_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct insert_data *idata)
 {
-  char *ptr_values, *ptr_where, *ptr_mv;
+  char *ptr_values, *ptr_where, *ptr_mv, *ptr_set;
   int num=0, ret=0, have_flows=0, len=0;
 
   if (idata->mv.last_queue_elem) {
@@ -284,18 +284,22 @@ int SQLI_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct inser
   /* constructing sql query */
   ptr_where = where_clause;
   ptr_values = values_clause; 
+  ptr_set = set_clause;
   memset(where_clause, 0, sizeof(where_clause));
   memset(values_clause, 0, sizeof(values_clause));
-  while (num < idata->num_primitives) {
+  memset(set_clause, 0, sizeof(set_clause));
+
+  for (num = 0; num < idata->num_primitives; num++)
     (*where[num].handler)(cache_elem, idata, num, &ptr_values, &ptr_where);
-    num++;
-  }
+
+  for (num = 0; set[num].type; num++)
+    (*set[num].handler)(cache_elem, idata, num, &ptr_set, NULL);
   
   /* sending UPDATE query */
   if (!config.sql_dont_try_update) {
-    if (have_flows) ret = snprintf(sql_data, sizeof(sql_data), update_clause, cache_elem->packet_counter, cache_elem->bytes_counter, cache_elem->flows_counter);
-    else ret = snprintf(sql_data, sizeof(sql_data), update_clause, cache_elem->packet_counter, cache_elem->bytes_counter);
-    strncpy(sql_data+ret, where_clause, SPACELEFT_LEN(sql_data, ret));
+    strncpy(sql_data, update_clause, SPACELEFT(sql_data));
+    strncat(sql_data, set_clause, SPACELEFT(sql_data));
+    strncat(sql_data, where_clause, SPACELEFT(sql_data));
 
     ret = sqlite3_exec(db->desc, sql_data, NULL, NULL, NULL);
     if (ret) goto signal_error; 
@@ -404,7 +408,7 @@ void SQLI_cache_purge(struct db_cache *queue[], int index, struct insert_data *i
     strftime_same(lock_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
     if (config.sql_table_schema && idata->new_basetime) sql_create_table(bed.p, idata);
   }
-  strncat(update_clause, set_clause, SPACELEFT(update_clause));
+  // strncat(update_clause, set_clause, SPACELEFT(update_clause));
 
   (*sqlfunc_cbr.lock)(bed.p); 
 
@@ -437,7 +441,7 @@ void SQLI_cache_purge(struct db_cache *queue[], int index, struct insert_data *i
 
 int SQLI_evaluate_history(int primitive)
 {
-  if (config.sql_history) {
+  if (config.sql_history || config.nfacctd_sql_log) {
     if (primitive) {
       strncat(insert_clause, ", ", SPACELEFT(insert_clause));
       strncat(values[primitive].string, ", ", sizeof(values[primitive].string));
@@ -465,7 +469,7 @@ int SQLI_evaluate_history(int primitive)
 
 int SQLI_compose_static_queries()
 {
-  int primitives=0, have_flows=0;
+  int primitives=0, set_primitives=0, have_flows=0;
 
   if (config.what_to_count & COUNT_FLOWS || (config.sql_table_version >= 4 && !config.sql_optimize_clauses)) {
     config.what_to_count |= COUNT_FLOWS;
@@ -494,18 +498,42 @@ int SQLI_compose_static_queries()
 
   /* "UPDATE ... SET ..." stuff */
   snprintf(update_clause, sizeof(update_clause), "UPDATE %s ", config.sql_table);
-#if defined HAVE_64BIT_COUNTERS
-  strncpy(set_clause, "SET packets=packets+%llu, bytes=bytes+%llu", SPACELEFT(set_clause));
-  if (have_flows) strncat(set_clause, ", flows=flows+%llu", SPACELEFT(set_clause));
-#else
-  strncpy(set_clause, "SET packets=packets+%lu, bytes=bytes+%lu", SPACELEFT(set_clause));
-  if (have_flows) strncat(set_clause, ", flows=flows+%lu", SPACELEFT(set_clause));
-#endif
-  if (config.sql_history) {
-    if (!config.sql_history_since_epoch)
-      strncat(set_clause, ", stamp_updated=DATETIME('now', 'localtime')", SPACELEFT(set_clause));
-    else
-      strncat(set_clause, ", stamp_updated=STRFTIME('%%s', 'now')", SPACELEFT(set_clause));
+
+  set_primitives = sql_compose_static_set(have_flows);
+
+  if (config.sql_history || config.nfacctd_sql_log) {
+    if (!config.nfacctd_sql_log) {
+      if (!config.sql_history_since_epoch) {
+	strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+	strncat(set[set_primitives].string, "stamp_updated=DATETIME('now', 'localtime')", SPACELEFT(set[set_primitives].string));
+	set[set_primitives].type = TIMESTAMP;
+	set[set_primitives].handler = count_noop_setclause_handler;
+	set_primitives++;
+      }
+      else {
+	strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+	strncat(set[set_primitives].string, "stamp_updated=STRFTIME('%%s', 'now')", SPACELEFT(set[set_primitives].string));
+	set[set_primitives].type = TIMESTAMP;
+	set[set_primitives].handler = count_noop_setclause_handler;
+	set_primitives++;
+      }
+    }
+    else {
+      if (!config.sql_history_since_epoch) {
+	strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+	strncat(set[set_primitives].string, "stamp_updated=DATETIME(%u, 'unixepoch', 'localtime')", SPACELEFT(set[set_primitives].string));
+	set[set_primitives].type = TIMESTAMP;
+	set[set_primitives].handler = count_timestamp_setclause_handler;
+	set_primitives++;
+      }
+      else {
+	strncpy(set[set_primitives].string, ", ", SPACELEFT(set[set_primitives].string));
+	strncat(set[set_primitives].string, "stamp_updated=%u", SPACELEFT(set[set_primitives].string));
+	set[set_primitives].type = TIMESTAMP;
+	set[set_primitives].handler = count_timestamp_setclause_handler;
+	set_primitives++;
+      }
+    }
   }
 
   return primitives;
@@ -589,7 +617,8 @@ void SQLI_init_default_values(struct insert_data *idata)
   /* Linking database parameters */
   if (!config.sql_db) config.sql_db = sqlite3_db;
   if (!config.sql_table) {
-    if (config.sql_table_version == 6) config.sql_table = sqlite3_table_v6;
+    if (config.sql_table_version == 7) config.sql_table = sqlite3_table_v7;
+    else if (config.sql_table_version == 6) config.sql_table = sqlite3_table_v6;
     else if (config.sql_table_version == 5) config.sql_table = sqlite3_table_v5;
     else if (config.sql_table_version == 4) config.sql_table = sqlite3_table_v4;
     else if (config.sql_table_version == 3) config.sql_table = sqlite3_table_v3;
