@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2006 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2007 by Paolo Lucente
 */
 
 /*
@@ -23,6 +23,8 @@
 #define ADD 0
 #define SUB 1
 
+#define DEBUG_TIMING 0
+
 /* prototypes */
 #if (!defined __UTIL_C)
 #define EXT extern
@@ -70,4 +72,18 @@ EXT void set_shadow_status(struct packet_ptrs *);
 EXT FILE *open_logfile(char *);
 
 EXT unsigned int str_to_addr(const char *, struct host_addr *);
+
+EXT struct packet_ptrs *copy_packet_ptrs(struct packet_ptrs *);
+EXT void free_packet_ptrs(struct packet_ptrs *);
 #undef EXT
+
+/* Timing Stuff */
+#if DEBUG_TIMING
+struct mytimer {
+  struct timeval t0;
+	struct timeval t1;
+};
+
+void start_timer(struct mytimer *);
+void stop_timer(struct mytimer *, const char *, ...);
+#endif
